@@ -1,5 +1,5 @@
 from ssl import SSLContext
-from typing import List, Optional, Tuple, Union
+from typing import List, Optional, Tuple
 
 from .._backends.auto import AsyncLock, AsyncSocketStream, AutoBackend
 from .._types import URL, Headers, Origin, TimeoutDict
@@ -10,8 +10,7 @@ from .base import (
     ConnectionState,
     NewConnectionRequired,
 )
-from .http2 import AsyncHTTP2Connection
-from .http11 import AsyncHTTP11Connection
+from .http import AsyncBaseHTTPConnection
 
 logger = get_logger(__name__)
 
@@ -21,24 +20,43 @@ class AsyncHTTPConnection(AsyncHTTPTransport):
         self,
         origin: Origin,
         http2: bool = False,
+        uds: str = None,
         ssl_context: SSLContext = None,
         socket: AsyncSocketStream = None,
+        local_address: str = None,
     ):
         self.origin = origin
         self.http2 = http2
+        self.uds = uds
         self.ssl_context = SSLContext() if ssl_context is None else ssl_context
         self.socket = socket
+        self.local_address = local_address
 
         if self.http2:
             self.ssl_context.set_alpn_protocols(["http/1.1", "h2"])
 
-        self.connection: Union[None, AsyncHTTP11Connection, AsyncHTTP2Connection] = None
+        self.connection: Optional[AsyncBaseHTTPConnection] = None
         self.is_http11 = False
         self.is_http2 = False
         self.connect_failed = False
         self.expires_at: Optional[float] = None
         self.backend = AutoBackend()
 
+    def __repr__(self) -> str:
+        http_version = "UNKNOWN"
+        if self.is_http11:
+            http_version = "HTTP/1.1"
+        elif self.is_http2:
+            http_version = "HTTP/2"
+        return f"<AsyncHTTPConnection http_version={http_version} state={self.state}>"
+
+    def info(self) -> str:
+        if self.connection is None:
+            return "Not connected"
+        elif self.state == ConnectionState.PENDING:
+            return "Connecting"
+        return self.connection.info()
+
     @property
     def request_lock(self) -> AsyncLock:
         # We do this lazily, to make sure backend autodetection always
@@ -82,9 +100,18 @@ class AsyncHTTPConnection(AsyncHTTPTransport):
         timeout = {} if timeout is None else timeout
         ssl_context = self.ssl_context if scheme == b"https" else None
         try:
-            return await self.backend.open_tcp_stream(
-                hostname, port, ssl_context, timeout
-            )
+            if self.uds is None:
+                return await self.backend.open_tcp_stream(
+                    hostname,
+                    port,
+                    ssl_context,
+                    timeout,
+                    local_address=self.local_address,
+                )
+            else:
+                return await self.backend.open_uds_stream(
+                    self.uds, hostname, ssl_context, timeout
+                )
         except Exception:
             self.connect_failed = True
             raise
@@ -95,11 +122,15 @@ class AsyncHTTPConnection(AsyncHTTPTransport):
             "create_connection socket=%r http_version=%r", socket, http_version
         )
         if http_version == "HTTP/2":
+            from .http2 import AsyncHTTP2Connection
+
             self.is_http2 = True
             self.connection = AsyncHTTP2Connection(
                 socket=socket, backend=self.backend, ssl_context=self.ssl_context
             )
         else:
+            from .http11 import AsyncHTTP11Connection
+
             self.is_http11 = True
             self.connection = AsyncHTTP11Connection(
                 socket=socket, ssl_context=self.ssl_context
@@ -111,7 +142,7 @@ class AsyncHTTPConnection(AsyncHTTPTransport):
             return ConnectionState.CLOSED
         elif self.connection is None:
             return ConnectionState.PENDING
-        return self.connection.state
+        return self.connection.get_state()
 
     def is_connection_dropped(self) -> bool:
         return self.connection is not None and self.connection.is_connection_dropped()
@@ -123,6 +154,10 @@ class AsyncHTTPConnection(AsyncHTTPTransport):
     async def start_tls(self, hostname: bytes, timeout: TimeoutDict = None) -> None:
         if self.connection is not None:
             logger.trace("start_tls hostname=%r timeout=%r", hostname, timeout)
-            await self.connection.start_tls(hostname, timeout)
+            self.socket = await self.connection.start_tls(hostname, timeout)
             logger.trace("start_tls complete hostname=%r timeout=%r", hostname, timeout)
-            self.socket = self.connection.socket
+
+    async def aclose(self) -> None:
+        async with self.request_lock:
+            if self.connection is not None:
+                await self.connection.aclose()
